#!/bin/ksh
# file: isdn_conf 
# setting up ISDN interfaces
#
# start: configure interface (without phonenumbers)
# up   : add phone + network
# down : del phone + network
# stop : remove interface ( + network )
# in   : only incoming connections
# out  : only outgoing connections
#
# (c) 1998 J. Hellmerichs-Friedrich

usage ()
{
cat << EOF
    Usage: $0 <action> [ <action> ... ]

    <action>    description
    -------------------------------------------------
     start      start  ISDN device (w/o network, tel)
     stop       remove ISDN device
     up         setup network, tel.-nr
     down       stop    "    ,  "
     dial       dial ISP
     hup|off    hangup phone
     in         delete all outgoing tel.-nr
     out          "    all incoming tel.-nr
     list       list ISDN parameters
     help|-h    this help
EOF
exit 1
}

case "$1" in
    help|-hi) usage;;
esac

# this script is called through a link, e.g as ippp0
# -> ISDN_DEV = ippp0; ISDN_DEV_NR = 0

ISDN_DEV=`basename $0`
ISDN_DEV_NR=`echo "$ISDN_DEV" | tr -d [a-zA-Z]`

# read (SuSE) configuration file
[ -r /etc/rc.config ] && . /etc/rc.config
[ "$I4L_START" = yes ] || exit 0

# read additional ISDN configuration file
[ -r /etc/isdn/isdn.config ] && . /etc/isdn/isdn.config

# SuSE configs are setup in /etc/rc.config like
# NETDEVICE_0=eth0
# NETDEVICE_1=ippp0
# NETDEVICE_2=isdn0
#
# we have to check if/which config definition is used for
# our interface (e.g. ippp0 uses _1 definitions)

CFG_NR=`set | grep NETDEV_ | grep $ISDN_DEV`
if [ -n "$CFG_NR" ]; then
    CFG_NR=`echo "$CFG_NR" | sed -e s/NETDEV_// -e s/=.*//`
else
    echo "Error: No configuration for $ISDN_DEV found."
    echo "       Define your settings in /etc/isdn/isdn.config"
    echo ""
    exit 1
fi

set_var ()
{
    eval variable="\$$2_$CFG_NR"
    test -n "$variable" && eval $1=\"$variable\"
}

#
# set config/default values
#
set_var MSN_ID I4L_LOCALMSN
if [ -z "$MSN_ID" ]; then
    echo "You must set a MSN/EAZ for $ISDN_DEV"
    echo "Edit /etc/rc.config or /etc/isdn/isdn.config and define 'I4L_LOCALMSN_$CFG_NR'"
    echo ""
    exit 0
fi
set_var MSN_IN I4L_REMOTE_IN
set_var MSN_OUT I4L_REMOTE_OUT
eval    MSN_IN2="\$I4L_REMOTE_IN1_$CFG_NR \$I4L_REMOTE_IN2_$CFG_NR"
eval    MSN_OUT2="\$I4L_REMOTE_OUT1_$CFG_NR \$I4L_REMOTE_OUT2_$CFG_NR"

set_var IFCONFIG IFCONFIG
set_var NETWORK NETWORK
set_var GATEWAY GATEWAY

set_var DEBUG=$I4L_DEBUG
set_var SECURE I4L_SECURE
set_var DIALMAX I4L_DIALMAX
set_var IDLETIME I4L_IDLETIME

set_var IHUP I4L_IHUP
set_var CBHUP I4L_CBHUP
set_var CBDELAY I4L_CBDELAY
set_var CALLBACK I4L_CALLBACK
set_var CHARGEHUP I4L_CHARGEHUP

set_var PROTOCOL I4L_PROTOCOL
set_var ENCAP I4L_ENCAP
set_var L2 I4L_L2
set_var L3 I4L_L3

set_var BIND I4L_BIND
set_var SLAVE I4L_SLAVE

# these functions ignore empty settings
# set_isdn()  call isdnctrl for $ISDN_DEV
# add_phone() add a phone number
# del_phone() del phone numbers

set_isdn ()
{
  isdn_dev="$ISDN_DEV"
  if [ "$1" = slave ]; then
      isdn_dev="$SLAVE"
      shift
  fi
  param="$1"
  shift
  values=$*

  if [ -n "$param" -a -n "$values" ]; then
      /sbin/isdnctrl $param $isdn_dev $values
  fi
}

add_phone ()
{
  isdn_dev="$ISDN_DEV"
  slave=""
  if [ "$1" = slave ]; then
      isdn_dev="$SLAVE"
      slave=slave
      shift
  fi
  dir="$1"
  case $dir in
    in) nrs="Incoming:"
        ;;
    out) nrs="Outgoing:"
        ;;
  esac
  shift
  numbers="$*"

  for number in $numbers; do
    [ -z "`/sbin/isdnctrl list $isdn_dev | grep "$nrs" | sed -e s/.*$nrs// | grep $number`" ] && \
      set_isdn $slave addphone $dir $number
  done
}

del_phone ()
{
  isdn_dev="$ISDN_DEV"
  slave=""
  if [ "$1" = slave ]; then
      isdn_dev="$SLAVE"
      slave=slave
      shift
  fi
  dir="$1"
  case $dir in
    in) nrs="Incoming:"
        ;;
    out) nrs="Outgoing:"
        ;;
  esac
  shift
  numbers="$*"
  [ "$numbers" = all ] && \
     numbers=`/sbin/isdnctrl list $isdn_dev | grep "$nrs" | sed -e s/.*$nrs//`

  for number in $numbers; do
    [ -n "`/sbin/isdnctrl list $isdn_dev | grep "$nrs" | grep $number`" ] && \
      set_isdn $slave delphone $dir $number
  done
}

#
# Loop over all parameters
#
while [ $# -gt 0 ]; do
  case "$1" in
    start)
        #
        # setup interface
        #
        echo "Starting isdn4linux on device $ISDN_DEV"
        set_isdn addif " " 2> /dev/null
        cat /proc/net/dev | grep $ISDN_DEV > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            echo "$0: $ISDN_DEV doesn't exist in /proc/net/dev."
            echo "Check ISDN hardware configuration."
            echo ""
            exit 1
        fi
        set_isdn eaz        $MSN_ID
        set_isdn dialmax    $DIALMAX
        set_isdn callback   $CALLBACK
        set_isdn cbdelay    $CBDELAY
        set_isdn cbhup      $CBHUP
        set_isdn chargehup  $CHARGEHUP
        set_isdn ihup       $IHUP
        set_isdn huptimeout $IDLETIME
        set_isdn l2_prot    $L2
        set_isdn l3_prot    $L3
        set_isdn encap      $ENCAP
        set_isdn protocol   $PROTOCOL
        set_isdn secure     $SECURE
        set_isdn bind       $BIND
        if [ -n "$SLAVE" ]; then
            set_isdn       addslave   $SLAVE
            set_isdn slave eaz        $MSN_ID
            set_isdn slave l2_prot    $L2
            set_isdn slave l3_prot    $L3
            set_isdn slave encap      $ENCAP
            set_isdn slave huptimeout $IDLETIME
            set_isdn slave secure     $SECURE
            set_isdn slave chargehup  $CHARGEHUP
        fi
        ;;

    up)
        #
        # add phone numbers
        #
        add_phone in  $MSN_IN  $MSN_IN2
        add_phone out $MSN_OUT $MSN_OUT2

        if [ -n "$SLAVE" ]; then
            add_phone slave in  $MSN_IN  $MSN_IN2
            add_phone slave out $MSN_OUT $MSN_OUT2
        fi
        #
        # setup network
        #
        echo -n "Setting up network device $ISDN_DEV, "
        /sbin/ifconfig $ISDN_DEV $IFCONFIG
        if [ -n "$GATEWAY" ]; then
                /sbin/route add $GATEWAY $ISDN_DEV
        fi
        if [ -n "$NETWORK" ]; then
                if [ -n "$GATEWAY" ]; then
                        /sbin/route add $NETWORK gw $GATEWAY $ISDN_DEV
                else
                        /sbin/route add $NETWORK $ISDN_DEV
                fi
        fi
        case $ISDN_DEV in
            ippp*)
                # start ppp-daemon
                OPTIONFILE=/etc/ppp/options.$ISDN_DEV
                PIDFILE=/var/run/ipppd.$ISDN_DEV.pid
                # is there running an old ipppd?
                if test -s $PIDFILE; then
                    echo ""
                    echo "Warning: $PIDFILE exists"
                    echo "          is there an old ipppd running?"
                    echo "Warning: kill -15 `cat $PIDFILE`"
                    kill -15 `cat $PIDFILE`
                    if [ -n "`ps ax | grep $PIDFILE | grep -v grep`" ]; then
                        # ganz schoen hartnaeckig :-(
                        echo "Warning: kill -9 `cat $PIDFILE`"
                        kill -9 `cat $PIDFILE`
                    fi
                    pid="`ps ax | grep $PIDFILE | cut -b-5`"
                    if [ -n "$pid" ]; then
                        kill -15 "$pid"
                        sleep 3
                        if [ -n "`ps ax | grep $pid | grep -v grep`" ]; then
                            # ganz schoen hartnaeckig :-(
                            kill -9 $pid
                        fi
                    fi
                fi
                echo "start ippp-daemon"
                if [ -r "/etc/ppp/options.$ISDN_DEV" ]; then
                    /sbin/ipppd pidfile $PIDFILE file $OPTIONFILE
                else
                    /sbin/ipppd pidfile $PIDFILE $ISDN_DEV
                fi
                if [ "$I4L_PPPBIND" != "no" ]; then
                    ippp_nr=`echo $ISDN_DEV | cut -b5-`
                    isdnctrl pppbind $ISDN_DEV $ippp_nr
                    if [ -n "$SLAVE" ]; then
                        ippp_nr=`echo $SLAVE | cut -b5-`
                        isdnctrl pppbind $SLAVE $ippp_nr
                    fi
                fi
                ;;
        esac
        echo ""
        ;;

    stop)
        #
        # stop network
        #
        case $ISDN_DEV in
        ippp*)
            PIDFILE=/var/run/ipppd.$ISDN_DEV.pid
            [ -s "$PIDFILE" ] && kill -15 `cat $PIDFILE`
            if [ -s "$PIDFILE" ]; then
               echo "Warning: kill -9 `cat $PIDFILE`"
               kill -9 `cat $PIDFILE`
               [ -r $PIDFILE ] && rm $PIDFILE 2>/dev/null
            fi
            ;;
        *)
            ;;
        esac
        /sbin/ifconfig $ISDN_DEV down
        #
        # remove interface
        #
        set_isdn hangup " " 2>/dev/null
        set_isdn delif  " " 2>/dev/null
        ;;

    down)
        #
        # stop network
        #
        case $ISDN_DEV in
        ippp*)
            PIDFILE="/var/run/ipppd.$ISDN_DEV.pid"
            echo "Stop ipppd (`cat $PIDFILE 2> /dev/null`)"
            [ -s "$PIDFILE" ] && kill -15 `cat $PIDFILE`
            if [ -s "$PIDFILE" ]; then
               sleep 2
               echo "Warning: kill -9 `cat $PIDFILE`"
               kill -9 `cat $PIDFILE` 2>/dev/null
               [ -r $PIDFILE ] && rm $PIDFILE 2>/dev/null
            fi
            ;;
          *)
            ;;
        esac
        /sbin/ifconfig $ISDN_DEV down
        #
        # del phone number
        #
        set_isdn hangup " "
        del_phone in  all
        del_phone out all
        ;;

    in)
        echo "only incoming calls on $ISDN_DEV"
        #
        # del out-phone number / add in-phone number
        #
        add_phone in  $MSN_IN  $MSN_IN2
        del_phone out all
        if [ -n "$SLAVE" ]; then
            add_phone slave in  $MSN_IN  $MSN_IN2
            del_phone slave out all
        fi
        ;;

    list|dial)
        set_isdn "$1" " "
        ;;

    off|hup|hangup)
        set_isdn hangup " "
        ;;
    out)
        echo "only outgoing calls on $ISDN_DEV"
        #
        # del in-phone number / add out-phone number
        #
        del_phone in  all
        add_phone out $MSN_OUT $MSN_OUT2
        if [ -n "$SLAVE" ]; then
            del_phone slave in  all
            add_phone slave out $MSN_OUT $MSN_OUT2
        fi
        ;;

    *)
        echo "Usage: $0 {start|stop|up|down|in|out|off|hup|hangup|list}"
        exit 1
        ;;
  esac
  shift
done
exit 0

